#!/bin/sh

### COPYLEFT
# Author: Sedat Dilek <sedat.dilek@gmail.com>
# Last changed: 28-Jan-2021

export LANG=C
export LC_ALL=C

BASE_DIR=$(pwd)
LINUX_DIR="${BASE_DIR}/git"
DEBUGFS_PGO_DIR="/sys/kernel/debug/pgo"

KVER="5.10"
##LLVM_MAJOR_VER="11"
LLVM_MAJOR_VER="12"

##LLVM_TOOLCHAIN_PATH="/usr/lib/llvm-${LLVM_MAJOR_VER}/bin"
LLVM_TOOLCHAIN_PATH="/opt/llvm-toolchain/bin"
if [ -d ${LLVM_TOOLCHAIN_PATH} ]; then
   export PATH="${LLVM_TOOLCHAIN_PATH}:${PATH}"
fi

MAKE="make"
MAKE_OPTS="V=1 -j4 HOSTCC=clang HOSTCXX=clang++ HOSTLD=ld.lld CC=clang LD=ld.lld LLVM=1 LLVM_IAS=1"

# XXX: Drop usage of perf - might have an impact on PROFDATA?
##PERF="/usr/bin/perf_${KVER}"
##PERF_OPTS="stat"

LLVM_PROFDATA="llvm-profdata-${LLVM_MAJOR_VER}"

echo ""
echo "Base dir .......... ${BASE_DIR}"
echo "Linux dir ......... ${LINUX_DIR}"
echo "DebugFS PGO dir ... ${DEBUGFS_PGO_DIR}"
echo "LLVM TC path ...... ${LLVM_TOOLCHAIN_PATH}"
echo "User's PATH env ... ${PATH}"
echo "make .............. ${MAKE}"
echo "make opts ......... ${MAKE_OPTS}"
echo "perf .............. ${PERF}"
echo "perf opts ......... ${PERF_OPTS}"
echo "llvm-profdata ..... ${LLVM_PROFDATA}"
echo ""

cd $LINUX_DIR

echo "XXX: Run 'make distclean' ..."
$MAKE distclean

echo "XXX: Create Linux x86-64 defconfig ..."
$MAKE $MAKE_OPTS defconfig

echo "XXX: Reset PGO profiling counters ..."
sudo -i -- sh -c "echo 1 > ${DEBUGFS_PGO_DIR}/reset"
sudo -i -- sh -c "ls -alh ${DEBUGFS_PGO_DIR}"

echo "XXX: Build Linux x86-64 defconfig ..."
# XXX: Enable some custom Kconfigs
##scripts/config -e PGO_CLANG
$PERF $PERF_OPTS $MAKE $MAKE_OPTS

echo "XXX: Copy PGO profdata ..."
sudo -i -- sh -c "cp -av ${DEBUGFS_PGO_DIR}/profraw ${BASE_DIR}/vmlinux.profraw"
sudo -i -- sh -c "chown dileks:dileks ${BASE_DIR}/vmlinux.profraw"
sudo -i -- sh -c "ls -alh ${BASE_DIR}/vmlinux.profraw"

echo "XXX: Merge PGO profdata ..."
cd ${BASE_DIR}
$LLVM_PROFDATA merge --output=vmlinux.profdata vmlinux.profraw
ls -alh vmlinux.profdata vmlinux.profraw
